// ----------------------------------------------------------------------------
//
//  Copyright (C) 2015-2017 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


//-----------------------------------------------------------------------------------
// Common definitions
//-----------------------------------------------------------------------------------


#include "sn3dpan3.h"
#include "zrotator3.h"
#include "sn3dhtop3.h"
#include "convert3.h"
#include "ambixfm3.h"


static const char* author = "Fons Adriaensen <fons@linuxaudio.org>";
static const char* license = "GPL";


static void pconnect (LADSPA_Handle H, unsigned long port, LADSPA_Data *data)
{
    ((LadspaPlugin *)H)->setport (port, data);
}

static void activate (LADSPA_Handle H)
{
    ((LadspaPlugin *)H)->active (true);
}

static void runplugin (LADSPA_Handle H, unsigned long k)
{
    ((LadspaPlugin *)H)->runproc (k, false);
}
/*
  static void runadding (LADSPA_Handle H, unsigned long k)
  {
  ((LadspaPlugin *)H)->runproc (k, true);
  }

  static void setadding (LADSPA_Handle H, LADSPA_Data gain)
  {
  ((LadspaPlugin *)H)->setgain (gain);
  }
*/
static void deactivate (LADSPA_Handle H)
{
    ((LadspaPlugin *)H)->active (false);
}

static void cleanup (LADSPA_Handle H)
{
    delete (LadspaPlugin *) H;
}


//-----------------------------------------------------------------------------------
//  LADSPA dlsym interface
//-----------------------------------------------------------------------------------


#define NMODS 8


static const LADSPA_Descriptor moddescr [NMODS] =
{
    {
	3703,
	"SN3D-panner3",
	LADSPA_PROPERTY_HARD_RT_CAPABLE,
        "Degree 3 AMB panner (Mono -> SN3D)",
	author,
	license,
	Ladspa_Sn3dpan3::NPORT,
	Ladspa_Sn3dpan3::portdescr,
	Ladspa_Sn3dpan3::portnames,
	Ladspa_Sn3dpan3::porthints,
	0,
	Ladspa_Sn3dpan3::create,
	pconnect,
	activate,
	runplugin,
	0,
	0,
	deactivate,
	cleanup
    },  
    {
	3704,
        "SN3D-panner3s",
	LADSPA_PROPERTY_HARD_RT_CAPABLE,
        "Degree 3 AMB panner (Stereo > SN3D)",
	author,
	license,
	Ladspa_Sn3dpan3s::NPORT,
	Ladspa_Sn3dpan3s::portdescr,
	Ladspa_Sn3dpan3s::portnames,
	Ladspa_Sn3dpan3s::porthints,
	0,
	Ladspa_Sn3dpan3s::create,
	pconnect,
	activate,
	runplugin,
	0,
	0,
	deactivate,
	cleanup
    },
    {
	3705,
        "AMB-Z-rotator3",
	LADSPA_PROPERTY_HARD_RT_CAPABLE,
        "Degree 3 AMB Z-rotator",
	author,
	license,
	Ladspa_Zrotator3::NPORT,
	Ladspa_Zrotator3::portdescr,
	Ladspa_Zrotator3::portnames,
	Ladspa_Zrotator3::porthints,
	0,
	Ladspa_Zrotator3::create,
	pconnect,
	activate,
	runplugin,
	0,
	0,
	deactivate,
	cleanup
    },  
    {
	3706,
        "SN3D-HtoP3",
	LADSPA_PROPERTY_HARD_RT_CAPABLE,
        "Degree 3 H to P converter (SN3D)",
	author,
	license,
	Ladspa_Sn3dhtop3::NPORT,
	Ladspa_Sn3dhtop3::portdescr,
	Ladspa_Sn3dhtop3::portnames,
	Ladspa_Sn3dhtop3::porthints,
	0,
	Ladspa_Sn3dhtop3::create,
	pconnect,
	activate,
	runplugin,
	0,
	0,
	deactivate,
	cleanup
    },
    {
	3707,
        "SN3D-to-N3D3",
	LADSPA_PROPERTY_HARD_RT_CAPABLE,
        "Degree 3 SN3D to N3D convertor",
	author,
	license,
	Ladspa_Hoa3convert::NPORT,
	Ladspa_Hoa3convert::portdescr,
	Ladspa_Hoa3convert::portnames,
	Ladspa_Hoa3convert::porthints,
	0,
	Ladspa_Hoa3convert::create_sn3d_to_n3d,
	pconnect,
	activate,
	runplugin,
	0,
	0,
	deactivate,
	cleanup
    },  
    {
	3708,
        "N3D-to-SN3D3",
	LADSPA_PROPERTY_HARD_RT_CAPABLE,
        "Degree 3 N3D to SN3D convertor",
	author,
	license,
	Ladspa_Hoa3convert::NPORT,
	Ladspa_Hoa3convert::portdescr,
	Ladspa_Hoa3convert::portnames,
	Ladspa_Hoa3convert::porthints,
	0,
	Ladspa_Hoa3convert::create_n3d_to_sn3d,
	pconnect,
	activate,
	runplugin,
	0,
	0,
	deactivate,
	cleanup
    },  
    {
	3709,
        "SN3D-to-FuMa3",
	LADSPA_PROPERTY_HARD_RT_CAPABLE,
        "Degree 3 SN3D to FuMa converter",
	author,
	license,
	Ladspa_Hoa3convert::NPORT,
	Ladspa_Hoa3convert::portdescr,
	Ladspa_Ambixtofuma3::portnames,
	Ladspa_Hoa3convert::porthints,
	0,
	Ladspa_Ambixtofuma3::create,
	pconnect,
	activate,
	runplugin,
	0,
	0,
	deactivate,
	cleanup
    },  
    {
	3710,
        "FuMa-to-SN3D3",
	LADSPA_PROPERTY_HARD_RT_CAPABLE,
        "Degree 3 FuMa to SN3D converter",
	author,
	license,
	Ladspa_Hoa3convert::NPORT,
	Ladspa_Hoa3convert::portdescr,
	Ladspa_Fumatoambix3::portnames,
	Ladspa_Hoa3convert::porthints,
	0,
	Ladspa_Fumatoambix3::create,
	pconnect,
	activate,
	runplugin,
	0,
	0,
	deactivate,
	cleanup
    }  
};


extern "C" const LADSPA_Descriptor *ladspa_descriptor (unsigned long i)
{
    if (i >= NMODS) return 0;
    return moddescr + i;
}


